﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using UnityEditor;
using UnityEngine;
using Debug = UnityEngine.Debug;


[ExecuteInEditMode]
public class StreamingModelPreview : MonoBehaviour
{
#if UNITY_EDITOR
    private CancellationTokenSource m_linkedCancellationTokenSource;
    public enum PreviewQuality
    {
        High = 0,
        Normal = 1,
        Low = 2,
        VeryLow = 3,
    }

    public PreviewQuality previewQuality = PreviewQuality.Low;

    private Sos.ScenePreview m_scenePreview;
    private IDestroyTokenWrapper m_destroyTokenWrapper;
    private PlayModeStateChange m_playModeState;
    private StreamingModel m_model;

    private void Awake()
    {
        if(m_destroyTokenWrapper == null) m_destroyTokenWrapper = DestroyTokenWrapperFactory.CreateIDestroyTokenWrapper(this);
        m_model = GetComponent<StreamingModel>();
    }

    private void Start()
    {
        EditorApplication.wantsToQuit += WantsToQuitCallback;
        EditorApplication.quitting += OnDestroy;
    }
    
    bool WantsToQuitCallback()
    {
        OnDestroy();
        return true;
    }
    private void OnEnable()
    {
        EditorApplication.playModeStateChanged -= PlayModeStateChanged;
        EditorApplication.playModeStateChanged += PlayModeStateChanged;
        UnityEditor.SceneView.duringSceneGui -= MyOnSceneGUI;
        UnityEditor.SceneView.duringSceneGui += MyOnSceneGUI;

        CheckModel();
    }

    private void OnDisable()
    {
        EditorApplication.playModeStateChanged -= PlayModeStateChanged;
        UnityEditor.SceneView.duringSceneGui -= MyOnSceneGUI;
        m_destroyTokenWrapper?.CancelActiveToken();
        m_scenePreview?.Dispose();
        m_scenePreview = null;
    }

    private void OnDestroy()
    {
        EditorApplication.wantsToQuit -= WantsToQuitCallback;
        EditorApplication.quitting -= OnDestroy;
        UnityEditor.SceneView.duringSceneGui -= MyOnSceneGUI;
        try
        {
            m_destroyTokenWrapper?.CancelActiveToken();
        }
        catch
        {
        }

        m_destroyTokenWrapper?.Dispose();
        m_destroyTokenWrapper= null;
        try
        {
            m_linkedCancellationTokenSource?.Cancel();
        }
        catch
        {
        }

        m_linkedCancellationTokenSource?.Dispose();
        m_linkedCancellationTokenSource = null;
        
        m_scenePreview?.Dispose();
        m_scenePreview = null;
    }

    private void Update()
    {
        if (ShouldUpdate())
        {
            CheckModel();
            if(m_scenePreview != null)
                m_scenePreview.Draw();
        }
    }

    private CancellationToken GetCancellationToken(StreamingModel model)
    {
        CancellationToken cancellationToken = default;
        if (m_destroyTokenWrapper == null) //this should be called in awake, and isn't always there
        {
            m_destroyTokenWrapper = DestroyTokenWrapperFactory.CreateIDestroyTokenWrapper(this);
        }
                
        if(model != null && model.DestroyTokenWrapper != null)
        {
            //cancel if this or the model is destroyed
            if(m_linkedCancellationTokenSource == null){
                m_linkedCancellationTokenSource = CancellationTokenSource.CreateLinkedTokenSource(m_destroyTokenWrapper.CancellationToken,model.DestroyTokenWrapper.CancellationToken); // cancel if either monobehaviour is destroyed
                /* //not sure about this, but something like this may be useful in some situations?
                 EditorApplication.playModeStateChanged += (PlayModeStateChange state) =>
                {
                    //maybe any transition? but not sure about lifecycle of this object relative to entering
                    //if (state == PlayModeStateChange.ExitingPlayMode || state == PlayModeStateChange.ExitingEditMode || state == PlayModeStateChange.EnteredPlayMode || state == PlayModeStateChange.EnteredEditMode)
                    if (state == PlayModeStateChange.ExitingPlayMode || state == PlayModeStateChange.ExitingEditMode)
                        m_linkedCancellationTokenSource.Cancel();
                };
                */
            }
            cancellationToken = m_linkedCancellationTokenSource.Token;
        }
        else
        {
            cancellationToken = m_destroyTokenWrapper.CancellationToken;
        }
        return cancellationToken;
    }
    private void CheckModel()
    {
        HasBeenAWhileSinceReload(); //call for side effect of updating timer unconditionally for now
        if (!ShouldUpdate()) return;
        if (m_model == null) m_model = GetComponent<StreamingModel>();

        if (m_model == null)
        {
            m_scenePreview?.Dispose();
            m_scenePreview = null;
            return;
        }

        if (m_scenePreview != null && (
                m_scenePreview.url != m_model.sourceUrl
             || m_scenePreview.qualityLevel != (int)previewQuality
             || m_scenePreview.hashCode != m_model.hashCode))
        {
            m_scenePreview?.Dispose();
            m_scenePreview = null;
        }
        if (m_scenePreview == null)
        {
            try
            {
                CancellationToken cancellationToken = GetCancellationToken(m_model);
                m_scenePreview = new Sos.ScenePreview(m_model.sourceUrl, m_model.hashCode, m_model.transform, (int)previewQuality, cancellationToken);
            }catch(Exception e)
            {
                Debug.LogError("SCENE PREVIEW FAILED TO BE CREATED");
                Debug.LogError(e);
            }
        }
    }

    private bool ShouldUpdate()
    {
        return isActiveAndEnabled && !EditorApplication.isPlayingOrWillChangePlaymode && !EditorApplication.isCompiling &&
               ((m_setEditorPlayMode && m_playModeState == PlayModeStateChange.EnteredEditMode) || HasBeenAWhileSinceReload());
    }

    private bool ModelWasCreatedAndIsCancelled()
    {
        return m_linkedCancellationTokenSource != null &&
               m_linkedCancellationTokenSource.Token.IsCancellationRequested;
    }

    public void MyOnSceneGUI(UnityEditor.SceneView scenevie)
    {
        if (m_scenePreview != null && ShouldUpdate() && !ModelWasCreatedAndIsCancelled())
        {
            m_scenePreview.Draw();
        }
    }

    private bool m_setEditorPlayMode = false;
    private bool m_DelayedEditorStartHappened = false;
    private readonly Stopwatch m_delayedStartStopwatch = new Stopwatch();
    float m_lastReloadTime = 0;
    private bool HasBeenAWhileSinceReload()
    {
        if(m_DelayedEditorStartHappened) return true;
        if (!m_delayedStartStopwatch.IsRunning)
        {
            m_delayedStartStopwatch.Start();
        }
        const float timeToWait = 1.0f;
        m_lastReloadTime = (float)m_delayedStartStopwatch.Elapsed.TotalSeconds;
        m_DelayedEditorStartHappened = m_delayedStartStopwatch.Elapsed.TotalSeconds > timeToWait; //should i just use a stopwatch here?
        if(m_DelayedEditorStartHappened)
        {
            m_delayedStartStopwatch.Stop();
        }
        return m_DelayedEditorStartHappened;
    }
    private void PlayModeStateChanged(PlayModeStateChange state)
    {
        m_setEditorPlayMode = false;
        m_playModeState = state;
        switch (state)
        {
            case PlayModeStateChange.EnteredPlayMode:
            case PlayModeStateChange.ExitingPlayMode:
            case PlayModeStateChange.ExitingEditMode:
                m_destroyTokenWrapper?.CancelActiveToken();
                m_scenePreview?.Dispose();
                m_scenePreview = null;
                break;
            case PlayModeStateChange.EnteredEditMode:
                m_setEditorPlayMode = true;
                m_DelayedEditorStartHappened = false;
                m_delayedStartStopwatch.Restart();
                break;
        }
    }
    
#endif
}
