#if UNITY_EDITOR
// SPDX-FileCopyrightText: 2023 Unity Technologies and the glTFast authors
// SPDX-License-Identifier: Apache-2.0

// Modifications made by HTC Corporation, 2024
using UnityEditor;
using UnityEngine;
//using static GLTFast.Materials.MaterialGenerator;
//using static GLTFast.Materials.BuiltInMaterialGenerator;

namespace Sos
{
    using Materials;
    class BuiltInShaderGUI : ShaderGUIBase
    {

        /// <summary>
        /// Subset of <see cref="StandardShaderMode"/> as not all configurations are supported
        /// </summary>
        enum BlendModeOption
        {
            Opaque,
            Cutout,
            Fade,
            Transparent,
        }

        UvTransform? m_UVTransform;

        public override void OnGUI(MaterialEditor materialEditor, UnityEditor.MaterialProperty[] properties)
        {
            if (materialEditor.target is Material material)
            {
                string current = material.GetTag(RenderTypeTag, false);
                BlendModeOption currentBlendMode = BlendModeOption.Opaque;

                switch (current)
                {
                    case "":
                    case OpaqueRenderType:
                        currentBlendMode = BlendModeOption.Opaque;
                        break;

                    case TransparentCutoutRenderType:
                        currentBlendMode = BlendModeOption.Cutout;
                        break;
                    case FadeRenderType:
                        currentBlendMode = BlendModeOption.Fade;
                        break;
                    case TransparentRenderType:
                        currentBlendMode = BlendModeOption.Transparent;
                        break;

                }
                GUILayout.BeginHorizontal();
                GUILayout.Label("Blend Mode");
                BlendModeOption blend = (BlendModeOption)EditorGUILayout.EnumPopup(currentBlendMode);
                GUILayout.EndHorizontal();

                if (blend != currentBlendMode)
                {
                    ConfigureBlendMode(material, blend);
                }

                m_UVTransform = TextureRotationSlider(
                    material,
                    m_UVTransform,
                    MaterialProperty.BaseColorTextureScaleTransform,
                    MaterialProperty.BaseColorTextureRotation,
                    true,
                    "Base Color Tex Rotation");
                if (m_UVTransform.HasValue)
                {
                    if (m_UVTransform.Value.rotation != 0)
                    {
                        material.EnableKeyword(TextureTransformKeyword);
                    }
                    else
                    {
                        material.DisableKeyword(TextureTransformKeyword);
                    }
                }

                if (GUI.changed)
                {
                    EditorUtility.SetDirty(material);
                }

                void EnsureKeywordForMap(string textureProperty, string keyword)
                {
                    if (material.HasProperty(textureProperty))
                    {
                        if (material.GetTexture(textureProperty) && !material.IsKeywordEnabled(keyword))
                            material.EnableKeyword(keyword);

                        if (!material.GetTexture(textureProperty) && material.IsKeywordEnabled(keyword))
                            material.DisableKeyword(keyword);
                    }
                }

                EnsureKeywordForMap("_MetallicGlossMap", "_METALLICGLOSSMAP");
                EnsureKeywordForMap("occlusionTexture", "_OCCLUSION");
            }

            base.OnGUI(materialEditor, properties);
        }

        static void ConfigureBlendMode(Material material, BlendModeOption mode)
        {
            switch (mode)
            {
                case BlendModeOption.Opaque:
                    SetOpaqueMode(material);
                    break;
                case BlendModeOption.Cutout:
                    SetAlphaModeMask(material, material.GetFloat(MaterialProperty.AlphaCutoff));
                    break;
                case BlendModeOption.Fade:
                    SetAlphaModeBlend(material);
                    break;
                case BlendModeOption.Transparent:
                    SetAlphaModeTransparent(material);
                    break;
            }
        }

        public static void SetAlphaModeMask(Material material, float alphaCutoff)
        {
            material.EnableKeyword(AlphaTestOnKeyword);
            material.SetInt(MaterialProperty.ZWrite, 1);
            material.DisableKeyword(k_AlphaPremultiplyOnKeyword);
            material.renderQueue = (int)UnityEngine.Rendering.RenderQueue.AlphaTest;  //2450
            material.SetFloat(MaterialProperty.AlphaCutoff, alphaCutoff);
            material.SetFloat(MaterialProperty.Mode, (int)StandardShaderMode.Cutout);
            material.SetOverrideTag(RenderTypeTag, TransparentCutoutRenderType);
            material.SetInt(MaterialProperty.SrcBlend, (int)UnityEngine.Rendering.BlendMode.One);
            material.SetInt(MaterialProperty.DstBlend, (int)UnityEngine.Rendering.BlendMode.Zero);
            material.DisableKeyword(k_AlphaBlendOnKeyword);
        }

        public static void SetOpaqueMode(Material material)
        {
            material.SetOverrideTag(RenderTypeTag, OpaqueRenderType);
            material.DisableKeyword(k_AlphaBlendOnKeyword);
            material.renderQueue = -1;
            material.SetInt(MaterialProperty.SrcBlend, (int)UnityEngine.Rendering.BlendMode.One);
            material.SetInt(MaterialProperty.DstBlend, (int)UnityEngine.Rendering.BlendMode.Zero);
            material.SetInt(MaterialProperty.ZWrite, 1);
            material.DisableKeyword(AlphaTestOnKeyword);
            material.DisableKeyword(k_AlphaPremultiplyOnKeyword);
        }

        public static void SetAlphaModeBlend(Material material)
        {
            material.SetFloat(MaterialProperty.Mode, (int)StandardShaderMode.Fade);
            material.SetOverrideTag(RenderTypeTag, FadeRenderType);
            material.EnableKeyword(k_AlphaBlendOnKeyword);
            material.SetInt(MaterialProperty.SrcBlend, (int)UnityEngine.Rendering.BlendMode.SrcAlpha);//5
            material.SetInt(MaterialProperty.DstBlend, (int)UnityEngine.Rendering.BlendMode.OneMinusSrcAlpha);//10
            material.SetInt(MaterialProperty.ZWrite, 0);
            material.DisableKeyword(k_AlphaPremultiplyOnKeyword);
            material.DisableKeyword(AlphaTestOnKeyword);
            material.renderQueue = (int)UnityEngine.Rendering.RenderQueue.Transparent;  //3000
        }

        public static void SetAlphaModeTransparent(Material material)
        {
            material.SetFloat(MaterialProperty.Mode, (int)StandardShaderMode.Fade);
            material.SetOverrideTag(RenderTypeTag, TransparentRenderType);
            material.EnableKeyword(k_AlphaPremultiplyOnKeyword);
            material.SetInt(MaterialProperty.SrcBlend, (int)UnityEngine.Rendering.BlendMode.One);//1
            material.SetInt(MaterialProperty.DstBlend, (int)UnityEngine.Rendering.BlendMode.OneMinusSrcAlpha);//10
            material.SetInt(MaterialProperty.ZWrite, 0);
            material.DisableKeyword(k_AlphaBlendOnKeyword);
            material.DisableKeyword(AlphaTestOnKeyword);
            material.renderQueue = (int)UnityEngine.Rendering.RenderQueue.Transparent;  //3000
        }
    }
}
#endif